@@ -94,6 +94,12 @@ int drop_privileges(struct mosquitto__config *config, bool temporary)
 	char err[256];
 	int rc;
 
+	const char *snap = getenv("SNAP_NAME");
+	if(snap && !strcmp(snap, "mosquitto")){
+		/* Don't attempt to drop privileges if running as a snap */
+		return MOSQ_ERR_SUCCESS;
+	}
+
 	if(geteuid() == 0){
 		if(config->user && strcmp(config->user, "root")){
 			pwd = getpwnam(config->user);
@@ -238,7 +244,7 @@ int main(int argc, char *argv[])
 
 	memset(&int_db, 0, sizeof(struct mosquitto_db));
 
-	net__init();
+	net__broker_init();
 
 	config__init(&int_db, &config);
 	rc = config__parse_args(&int_db, &config, argc, argv);
@@ -432,7 +438,7 @@ int main(int argc, char *argv[])
 	}
 
 	config__cleanup(int_db.config);
-	net__cleanup();
+	net__broker_cleanup();
 
 	return rc;
 }
