@@ -428,6 +428,7 @@ struct mosquitto__bridge{
 	int cur_address;
 	int address_count;
 	time_t primary_retry;
+	mosq_sock_t primary_retry_sock;
 	bool round_robin;
 	bool try_private;
 	bool try_private_accepted;
@@ -514,6 +515,8 @@ int send__suback(struct mosquitto *context, uint16_t mid, uint32_t payloadlen, c
 /* ============================================================
  * Network functions
  * ============================================================ */
+void net__broker_init(void);
+void net__broker_cleanup(void);
 int net__socket_accept(struct mosquitto_db *db, mosq_sock_t listensock);
 int net__socket_listen(struct mosquitto__listener *listener);
 int net__socket_get_address(mosq_sock_t sock, char *buf, int len);
@@ -596,6 +599,7 @@ int bridge__new(struct mosquitto_db *db, struct mosquitto__bridge *bridge);
 int bridge__connect(struct mosquitto_db *db, struct mosquitto *context);
 int bridge__connect_step1(struct mosquitto_db *db, struct mosquitto *context);
 int bridge__connect_step2(struct mosquitto_db *db, struct mosquitto *context);
+int bridge__connect_step3(struct mosquitto_db *db, struct mosquitto *context);
 void bridge__packet_cleanup(struct mosquitto *context);
 #endif
 
